######################################################################
##### Code for Day 4 of SEM Workshop
##### covering Composite Variables & Advanced Topics
#####
##### Jarrett E.K. Byrnes
#####
##### Last Modified 2/1/15
######################################################################

#Set your working Directory
#setwd("~/Dropbox/SEM/lavaan")


#################################################################################################################
# Composite Variables for Nonlinearities
#################################################################################################################


#Load your Data File
keeley<-read.csv("../Keeley_rawdata_select4.csv")


#is this a nonlinear relationship?
plot(rich ~ cover, data=keeley, cex.lab=1.5, cex.axis=1.3)

#compare models
linear<-lm(rich ~ cover, data=keeley)
nonlinear<-lm(rich ~ cover+I(cover^2), data=keeley)

library(AICcmodavg)
aictab(list(linear, nonlinear), c("linear", "squared"))

#plot the nonlinear result
curve(coef(nonlinear)[1] + coef(nonlinear)[2]*x + coef(nonlinear)[3]*x^2, add=T, col="red", lwd=4)
curve(coef(linear)[1] + coef(linear)[2]*x, add=T, col="blue", lwd=4)

#Step 1: Create a new nonlinear variable in the data
keeley<-within(keeley, coverSQ<-cover^2)

#Now, we can fit this with no composite
noCompModel <- 'rich ~ cover + coverSQ'

noCompFit <- sem(noCompModel, data=keeley)
summary(noCompFit, rsquare=T)

#Now, add a composite

compModel<-'

					coverEffect <~ cover + coverSQ

				  rich ~ 1*coverEffect'

compFit <- sem(compModel, data=keeley)

summary(compFit)

###
#Exercise: Abiotic
###

#how about abotic and fire severity?
plot(firesev ~ abiotic, data=keeley)

fireLinear<-lm(firesev ~ abiotic, data=keeley)
fireNonlinear<-lm(firesev ~ abiotic + I(abiotic^2), data=keeley)

aictab(list(fireLinear, fireNonlinear), c("linear", "squared"))

curve(coef(fireNonlinear)[1] + coef(fireNonlinear)[2]*x + coef(fireNonlinear)[3]*x^2, add=T, col="red", lwd=4)

keeley$abioticSQ <- keeley$abiotic^2



abioticCompModelBad<-'
                   abioticEffect <~ 0.4 * abiotic + abioticSQ

                  firesev ~ abioticEffect'

abioticCompModel<-'
                   abioticEffect <~ abiotic + abioticSQ

                  firesev ~ 1*abioticEffect'

abioticCompFit <- sem(abioticCompModel, data=keeley)

summary(abioticCompFit)

####
# Endogenous composite
####


endoCompModel<-'
                coverEffect <~ cover + coverSQ

                cover ~~ coverSQ
                age ~~ coverSQ

                cover ~ age
                rich ~ age + 1*coverEffect'


endoCompFit <- sem(endoCompModel, data=keeley, fixed.x=F)

summary(endoCompFit)

#################################################################################################################
# Multiple Categorical Predictors
#################################################################################################################

urchinData<-read.csv("../urchin_ex_sem.csv")


#Make treatment into a series of binary variables
source("../makeBinaryTreatments.R")
binTrt<-makeBinaryTreatments(urchinData, "treatment")

#add new columns to data frame
urchinData<-cbind(urchinData, binTrt)

#The observed variable model

urchinModel<-' 
  Feeding.rate.dry ~ MAPY + R
  GONAD_INDEX ~ MAPY + R + Feeding.rate.dry
'

urchinSEM<-sem(urchinModel, data=urchinData)

summary(urchinSEM, rsquare=T, standardized=T)

#now make a composite model


urchinCompositeModel<-' 
    Treatment <~ MAPY + R
  
    Feeding.rate.dry ~ 1*Treatment

    GONAD_INDEX ~ Treatment + Feeding.rate.dry
'

urchinCompositeSEM<-sem(urchinCompositeModel, data=urchinData)

summary(urchinCompositeSEM, rsquare=T, standardized=T)

anova(urchinSEM, urchinCompositeSEM)

########################
####Exercise
########################

urchinModelTest<-' 
  Feeding.rate.dry ~ MAPY + R
  TEST_CHANGE ~ MAPY + R + Feeding.rate.dry
'

urchinSEMT<-sem(urchinModelTest, data=urchinData)
inspect(urchinSEMT, "sample")


#So, fix test change!
urchinData$TEST_CHANGE_1000<-urchinData$TEST_CHANGE/1000



urchinModel2<-' 
  Feeding.rate.dry ~ MAPY + R
  TEST_CHANGE_10 ~ MAPY + R + Feeding.rate.dry
'

urchinSEM2<-sem(urchinModel2, data=urchinData)
summary(urchinSEM2)


urchinCompositeModel2<-' 
  Treatment <~ MAPY + R

  Feeding.rate.dry ~ 1*Treatment
  TEST_CHANGE_1000 ~Treatment + Feeding.rate.dry
'
urchinCompositeFit2<-sem(urchinCompositeModel2, data=urchinData)

summary(urchinCompositeFit2)


##############################################################################################################################
# Advanced Topics
##############################################################################################################################

#######
## total and indirect effects
#######



#Load your Data File
keeley<-read.csv("./Keeley_rawdata_select4.csv")


partialMedModel<-' firesev ~ af*age
                cover ~ fc*firesev + ac*age
          
                indirect := af*fc
                total := ac + (af*fc)

'

partialMedSEM<-sem(partialMedModel, 
                   data=keeley)

#look at coefficients
summary(partialMedSEM)

##############################################################################################################################
# D-Sep Revisited
##############################################################################################################################


#########
# D-Sep in lavaan
#########
#Full Mediation
distModel2 <- 'rich ~ abiotic  + hetero
              hetero ~ distance
              abiotic ~ distance'

distFit2 <- sem(distModel2, data=keeley)

source("./dsepTest.R")

dsepTest(distFit2)
dsepTest(distFit2, showall=T)


###########
#Survey
###########

#read in the data
cards<-read.csv("../cardainel_et_al_2009.csv")

#make a new nonlinear column
cards$logN2 <- cards$logN^2
cards$logN2Cen <- (cards$logN-mean(cards$logN))^2

cardModel<-'
SA ~ logN + logN2 + SR
logChl ~ SA + logN
logN ~~ logN2
'

#fixed.x=F in order to allow the dropping of paths between
#SR and logN and LogNcen2
cardFit <- sem(cardModel, data=cards, fixed.x=F, estimator="MLM")

library(lavaan.survey)

cardSrvy <- svydesign(ids=~Stream+Sub, data=cards, probs=~1)

fit.card.survey <- lavaan.survey(cardFit, cardSrvy, estimator = "MLM")
summary(fit.card.survey, standardized=T)
standardizedSolution(fit.card.survey)

######
#Shipley Exercise
######
Shipley<-read.table("./Shipley.dat")
head(Shipley)

shipSurv <- svydesign(ids =~site + tree, probs=~1, data=Shipley)

shipMod1 <- '
  Survival ~ Growth
  Growth ~ Date
  Date ~ DD
  DD ~ lat'

shipFit <- sem(shipMod1, data=Shipley, estimator="MLM")

shipCorrect <- lavaan.survey(shipFit, shipSurv, estimator="MLM")

summary(shipCorrect, standardized=T, rsq=T)


round(diag(vcov(shipFit)),3)
round(diag(vcov(shipCorrect)),3)

#########
# D-Sep from Shipley 2009
#The Example from the appendix
#########

Shipley<-read.table("./Shipley.dat")

pairs(Shipley)

###A brief survey detour
library(ggplot2)
qplot(DD, Date, data=Shipley, facets=~site, colour=tree) + theme_bw(base_size=16) 



####How would we fit paths?
#e.g., for DD -> lat
library(nlme)

#model with random intercept
#tree nested in site
Date_dd<-lme(Date~DD,data=Shipley,
             random=~1|site/tree,na.action=na.omit)



#Independence claim: (Date,lat)|{DD} 
fit1<-lme(Date~DD+lat,data=Shipley,random=~1|site/tree,na.action=na.omit)
summary(fit1)$tTable #9.373049e-01

#Independence claim: (Growth,lat)|{Date} 
fit2<-lme(Growth~Date+lat,data=Shipley,random=~1|site/tree,na.action=na.omit) 
summary(fit2)$tTable #3.836896e-01


#Independence claim: (Growth,DD)|{Date,lat} 
fit3<-lme(Growth~Date+lat+DD,data=Shipley,random=~1|site/tree,na.action=na.omit)
summary(fit3)$tTable #7.667083e-01

###need lme4 for the glmms
library(lme4)
qplot(Growth, Survival, data=Shipley, facets=~site, colour=tree) + theme_bw(base_size=16) 


#Independence claim with glmm (Live,lat)|{Growth}
fit4<-lmer(Live~Growth+lat+(1|site)+(1|tree),data=Shipley, na.action=na.omit, family=binomial(link="logit"))
coef(summary(fit4)) #2.791099e-01

#Independence claim with glmm (Live,DD)|{Growth,lat} 
fit5<-lmer(Live~Growth+lat+DD+(1|site)+(1|tree),data=Shipley,na.action=na.omit, family=binomial(link="logit"))
coef(summary(fit5)) #3.159286e-01

#Independence claim with glmm (Live,Date)|{DD,Growth} 
fit6<-lmer(Live~Growth +DD+Date+(1|site)+(1|tree),data=Shipley,na.action=na.omit, family=binomial(link="logit"))
summary(fit6)@coefs #1.519170e-01


#note, since we're logging things
#we can use log(a)+log(b) = log(a*b)
fisherC <- -2* log(9.373049e-01 * 3.836896e-01 * 7.667083e-01 * 2.791099e-01 * 3.159286e-01 * 1.519170e-01)

1-pchisq(fisherC, 2*6)


shipley2009.modlist = list(
  lme(DD~lat, random = ~1|site/tree, na.action = na.omit, 
      data = Shipley),
  
  lme(Date~DD, random = ~1|site/tree, na.action = na.omit, 
      data = Shipley),
  
  lme(Growth~Date, random = ~1|site/tree, na.action = na.omit, 
      data = Shipley),
  
  glmer(Live~Growth+(1|site)+(1|tree), 
        family=binomial(link = "logit"), data = Shipley) )

sem.fit(shipley2009.modlist, data=Shipley)


#each piece except last has 5 parameters - slope, 
# intercept, variance, and random variance for 
#slope & intercept, so, K=5*4-1

fisherC + 2*(5*4-1) 

sem.lavaan(shipley2009.modlist, data=Shipley)

#R^2 Values
sem.model.fits(shipley2009.modlist)


#######EXERCISE: Cardinale Example

#Model with transformed variable
cardModList <- list(
  lme(SA ~ logN + logN2 + SR, data=cards,
      random=~ 1|Stream/Sub, na.action=na.omit),
  
  lme(logChl ~ SA + logN, data=cards,
      random=~ 1|Stream/Sub, na.action=na.omit)

)

#Model with transformed variable
cardModListCent <- list(
  lme(SA ~ logN + logN2Cen + SR, data=cards,
      random=~ 1|Stream/Sub, na.action=na.omit),
  
  lme(logChl ~ SA + logN, data=cards,
      random=~ 1|Stream/Sub, na.action=na.omit)
  
)

#Model where variable is transformed internally
cardModListTrans <- list(
  lme(SA ~ logN + I(logN^2) + SR, data=cards,
      random=~ 1|Stream/Sub, na.action=na.omit),
  
  lme(logChl ~ SA + logN, data=cards,
      random=~ 1|Stream/Sub, na.action=na.omit)
)

#fit and evaluate the model
sem.fit(cardModList, data=cards)

#Get coefficients - note problem with internal transform
sem.coefs(cardModListTrans, data=cards, standardize="scale")
sem.coefs(cardModList, data=cards, standardize="scale")
sem.coefs(cardModListCent, data=cards, standardize="scale")

#Get R^2
sem.model.fits(cardModListTrans)
sem.model.fits(cardModListCent)


########################################################
####### Latent Growth Curve Models
########################################################

kfmTseries<-read.csv("./kfmTseries.csv")

gMod<-'
  Initial =~ 1*KelpT1 + 1*KelpT2 + 1*KelpT3 + 1*KelpT4
	Growth =~ 0*KelpT1 + 1*KelpT2 + 2*KelpT3 + 3*KelpT4
	KelpT2 ~~ KelpT1
	KelpT3 ~~ KelpT2
	KelpT4 ~~ KelpT3
	'
gFit<-growth(gMod, data=kfmTseries)

summary(gFit, rsquare=T)

gMod2<-'
  Initial =~ 1*KelpT1 + 1*KelpT2 + 1*KelpT3 + 1*KelpT4
  Growth =~ 0*KelpT1 + 1*KelpT2 + 2*KelpT3 + 3*KelpT4
	KelpT2 ~ a*KelpT1
	KelpT3 ~ a*KelpT2
	KelpT4 ~ a*KelpT3
	'
gFit2<-growth(gMod2, data=kfmTseries)

#compare
anova(gFit, gFit2)

summary(gFit2, rsquare=T)

nutrientGMod <-paste(gMod2, '
	
	Initial ~ Mean.Benthic.Temp
	Growth ~ Mean.Benthic.Temp

	', sep="\n")

nutrientGFit<-growth(nutrientGMod, data=kfmTseries)

summary(nutrientGFit)


##############
# Spatial
##############



#######
boreal <- read.table("./Boreality.txt", header=T)
source("./fitted_lavaan.R")
library(ggplot2)
qplot(x, y, data=boreal, size=Wet, color=NDVI) +
  theme_bw(base_size=18) + 
  scale_size_continuous("Index of Wetness", range=c(0,10)) + 
  scale_color_gradient("NDVI", low="lightgreen", high="darkgreen")

qplot(x, y, data=boreal, size=nTot, color=NDVI) +
  theme_bw() + 
  scale_size_continuous("Species Richness", range=c(0,10)) + 
  scale_color_gradient("NDVI", low="lightgreen", high="darkgreen")

qplot(x, y, data=boreal, size=T61, color=NDVI) +
  theme_bw() + 
  scale_size_continuous("Days Freezing", range=c(0,10)) + 
  scale_color_gradient("NDVI", low="lightgreen", high="darkgreen")



borModel <- '
NDVI ~ nTot + T61 + Wet 
nTot ~ T61
'

borFit <- sem(borModel, data=boreal, meanstructure=T)

summary(borFit)

#borRes <- as.data.frame(residuals(borFit, "casewise"))
borRes <- residuals_lavaan(borFit)


qplot(x, y, data=boreal, color=borRes$NDVI, size=I(5)) +
  theme_bw(base_size=17) + 
  scale_color_gradient("NDVI Residual", low="blue", high="yellow")


#Evaluate Spatial Residuals
#First create a distance matrix
library(ape)
distMat <- as.matrix(dist(cbind(boreal$x, boreal$y)))

#invert this matrix for weights
distsInv <- 1/distMat
diag(distsInv) <- 0


mi.ndvi <- Moran.I(borRes$NDVI, distsInv)
mi.ndvi


#New SE
#Where did we get the SE from?
sqrt(diag(vcov(borFit)))

n.ndvi <- nrow(boreal)*(1-mi.ndvi$observed)/(1+mi.ndvi$observed)

#New SE
ndvi.var <- diag(vcov(borFit))[1:3]

ndvi.se <- sqrt(ndvi.err*nrow(boreal)/n.ndvi)

ndvi.se


#compare to old
sqrt(diag(vcov(borFit)))[1:3]

#new z values
z <- coef(borFit)[1:3]/ndvi.se

2*pnorm(abs(z), lower.tail=F)


summary(borFit, standardized=T)

#Now try for nTot
Moran.I(borRes$nTot, distsInv)

#or.....try this easier solution from semTools
#library(semTools)
source("./lavSpatialCorrect.R")
lavSpatialCorrect(borFit, boreal$x, boreal$y)

