##################################################################################################################
#
# Prediction with SEM and Observed Variables
#
##################################################################################################################

#library
library(piecewiseSEM)
source("./predict_piecewiseSEM.R")

#Load your Data File
keeley<-read.csv("../Keeley_rawdata_select4.csv")


modList <- list(
  lm(rich ~ distance + abiotic + hetero, data=keeley),
  lm(abiotic ~ distance, data=keeley),
  lm(hetero ~ distance, data=keeley)
)


head(fitted_piecewiseSEM(modList))
residuals_piecewiseSEM(modList)
#
full_fitted <- predict_piecewiseSEM(modList, interval="confidence")

predict_piecewiseSEM(modList, newdata=data.frame(distance=c(30, 50)))
predict_piecewiseSEM(modList, newdata=data.frame(distance=c(30, 50)), interval="confidence")
pred_data <- data.frame(distance = seq(57,83,.01))
pred_data <- cbind(pred_data, predict_piecewiseSEM(modList, newdata=pred_data, interval="confidence"))
library(ggplot2)


ggplot(data=cbind(pred_data, pred_df_ci), mapping=aes(x=distance, y=rich_fit.2, 
                                                     ymin=rich_lwr.2, ymax=rich_upr.2)) +
  geom_line(color="red", lwd=2) +
  geom_ribbon(alpha=0.2) +
  xlab("Distance") +
  ylab("Predicted Species Richness\n") +
  theme_bw(base_size=18)


######
# Make a Model with Compound Paths
######
#library
library(lavaan)

totDistModel <- '
rich ~ a*distance + b*abiotic + c*hetero             
abiotic ~ b1*distance 
hetero ~ c1*distance  

rich ~ ia*1
abiotic ~ ib*1
hetero ~ ic*1

direct:=  a              
indirect:= b1*b + c1*c               
total:= b1*b + c1*c + a   
totalIntercept := ia + ib*b1 + ic*c1
'


#Fit the model and get coefficients from summary
totDistFit<-sem(totDistModel, data=keeley)

summary(totDistFit,  rsquare=T)

#Get new predictions
newDistance <- 57

#naieve prediction
newDistance*1 + -19.070

#adding parameter uncertainty
predDist <- rnorm(100, 1, 0.145)*newDistance + rnorm(100, -19.070, 9.683)

quantile(predDist)

#Let's get smome new data to plot!
new_data <- data.frame(distance = seq(57,83,.01))
new_data$predicted_richness <- new_data$distance*1 + -19.070

#Fill in the 95% CIs from the simulated data
new_data <- cbind(new_data, 
     t( sapply(new_data[,1], function(x)
        quantile( rnorm(100, 1, 0.145)*x + rnorm(100, -19.070, 9.683),
                  probs=c(0.025, 0.975)))
      ))

#Plot it!
library(ggplot2)
ggplot(data=new_data, mapping=aes(x=distance, y=predicted_richness, 
                                  ymin=`2.5%`, ymax=`97.5%`)) +
  geom_line(color="red", lwd=2) +
  geom_ribbon(alpha=0.2) +
  xlab("Distance") +
  ylab("Predicted Species Richness\n") +
  theme_bw(base_size=18)

##########OR

source("./fitted_lavaan.R")

new_data <- data.frame(distance = seq(57,83,.01))
pred_df <- predict_lavaan(totDistFit, newdata = new_data)
pred_df_ci <- predict_lavaan(totDistFit, newdata = new_data, simCI=TRUE, nsims=100)
head(pred_df_ci)

ggplot(data=cbind(new_data, pred_df_ci), mapping=aes(x=distance, y=rich, 
                                  ymin=rich_lower, ymax=rich_upper)) +
  geom_line(color="red", lwd=2) +
  geom_ribbon(alpha=0.2) +
  xlab("Distance") +
  ylab("Predicted Species Richness\n") +
  theme_bw(base_size=18)