#If you have time flowing in only one direction 
# (i.e., you do not have multiple measurements per time point)
adf1 <- data.frame(cbind(Time=1:10, Value=rnorm(10)))

#First, resort your data frame so that the rows are in order of time
#just in case it is out of order
adf1 <- adf1[sort(adf1$Time, index.return=T),]


#Second, use embed on the value of interest
#see ?embed for more...
adf1$lagValue <- c(NA, embed(adf1$Value,2)[,2])


#we can actually write a function to do this for 
#any data frame
lag <- function(x, time_var, value_var){
  x<- x[sort(x[[time_var]], index.return=T)$ix,]
  x$lag <- c(NA, embed(x[[value_var]],2)[,2])
  
  x
}

lag(adf1, "Time", "Value")

#Now, what if you have multiple measurements per time point
#at multiple sites?
adf <- data.frame(expand.grid(Site=c("A", "B"), Time=1:10))
adf$Value <- rnorm(nrow(adf))

#the easiest way to do this would be via plyr using the
#embed technique above.  


library(plyr)

adf <- ddply(adf, "Site", 
             function(my_data) lag(my_data, "Time", "Value"))
