predict_piecewiseSEM <- function(modList, newdata=NULL, ...){
  formulaList = get.formula.list(modList)
  lhs <- sapply(formulaList, function(x) as.character(x[[2]]))
  names(modList)<- lhs
  
  lhs_sorted <- colnames(get.sort.dag(formulaList))
  lhs_sorted <- lhs_sorted[which(lhs_sorted %in% lhs)]
  
  #Sort the modList
  modList <- modList[lhs_sorted]
  
  #If there is no new data, just get predict from each function  
  if(is.null(newdata)){
    ret <- lapply(modList, predict, ...)
    ret <- as.data.frame(do.call(cbind, ret))
  }else{
    
    #If there is new data...
    modData <- newdata
    ret <- data.frame(init = rep(NA, nrow(modData)))
    for(j in 1:length(modList)){
      pred <-  as.data.frame(predict(modList[[j]], newdata=modData, ...))
      ret <- cbind(ret, pred)
      modData <- cbind(modData, pred[,1])
      names(modData)[j+1] <- lhs_sorted[j]
    }
    ret <- ret[,-1]
  }
  
  #fix column names
  if(length(lhs)==ncol(ret)) {
    names(ret) <- lhs_sorted
  }else{
    lhs_new <- do.call(c,lapply(lhs_sorted, rep, ncol(ret)/length(lhs_sorted), simplify=TRUE))
    names(ret) <- paste(lhs_new, names(ret), sep="_")
  }
  
  return(ret)
}



fitted_piecewiseSEM <- function(modList){
  predict_piecewiseSEM(modList)
}

residuals_piecewiseSEM <- function(modList){
  formulaList = get.formula.list(modList)
  lhs <- sapply(formulaList, function(x) as.character(x[[2]]))
  names(modList)<- lhs
  ret <- lapply(modList, residuals)
  ret <- do.call(cbind, ret)
  return(ret)
}

# ######## EXAMPLES
# #first, make a list of models that is your 
# #SEM
# keeley<-read.csv("./Keeley_rawdata_select4.csv")
# 
# 
# modList <- list(
#   lm(abiotic ~ distance, data=keeley),
#   lm(rich ~ abiotic + hetero, data=keeley),
#   lm(hetero ~ distance, data=keeley))
#   
#   
#   
# fit <- sem.fit(modList, data=keeley)
# 
#   
# fit
#   
# 
# predict_piecewiseSEM(modList)
# fitted_piecewiseSEM(modList)
# residuals_piecewiseSEM(modList)
# 
# predict_piecewiseSEM(modList, interval="confidence")
# predict_piecewiseSEM(modList, newdata=data.frame(distance=c(30, 50)))
# predict_piecewiseSEM(modList, newdata=data.frame(distance=c(30, 50)), interval="confidence")

